<?php
/*
 * @author Ralph Bolton, July 2004, www.coofercat.com
 * @version 1.0
 * @copyright Copyright &copy; 2004 Ralph Bolton.
 * @license Tiki License
 * @see http://tikiwiki.org for Licensing information.
*/

// Initialisation
require_once('tiki-setup.php');
require_once('HTTP/Request.php');

/*
 * string urlt_perform_regexes(string url)
 * This function takes a URL (possibly relative or fully qualified,
 * of any scheme) and runs a series of configurable regexes on it.
 * The regexes have a flag associated with them that can cut short
 * processing. That is, stop processing any more regexes if a match
 * was found or not found, for example. What ever result is obtained
 * is returned.
 *
 * Regexes are found in the tiki_urlt_regex_in database table.
 *
 * @access private
*/
function perform_regexes($url)
{
	#error_log("URL to transform is $url");

	global $tikilib;

	// ADOdb style DB access.
	// We use a short cache here so that we keep the table in cache between
	// calls to the output filter (which are plentiful with each Tiki request).
	// There may be good reason to have a long cache here, at the cost of
	// reaction-to-configuration times. 
	if(($rs=$tikilib->db->CacheExecute(1,"select * from tiki_urlt_regex_in"))===false)
	{
		// Failed to read the database properly.
		error_log("urlt_in: Failed to Read database: " . $tikilib->db->ErrorMsg());
		return $url;
	}
	while ($array = $rs->FetchRow())
	{
		//error_log("ID: ".$array['regexid']." Left: ".$array['regex_left']." Right: ".$array['regex_right']." Flags: ".$array['regex_flags']." Continue: ".$array['continue_flag']);

		//$newurl=preg_replace('/'.$array['regex_left'].'/'.$array['regex_flags'], $array['regex_right'], $url);

		// First, see if the rule is disabled - if so, skip it.
                if($array['continue_flag'] == 3)
                {
                        // Rule disabled - skip it
                        //error_log("Skipping disabled rule " . $array['regexid']);
                        continue;
                }

		// We have to check first, replace second. We can't just attempt the replace
		// because there's no way to know if it was successful if you don't change
		// anything (which is possible - and likely!)
		//if($newurl == $url)
		if(!preg_match('/'.$array['regex_left'].'/'.$array['regex_flags'],$url))
		{
			# No change - no match!
			if($array['continue_flag'] == 2)
			{
				//error_log("Ceasing processing because flag = 2 (stop on no match)");
				break;
			}
		}
		else
		{
			# Changed - Match
			//$url=$newurl;
			$url=preg_replace('/'.$array['regex_left'].'/'.$array['regex_flags'], $array['regex_right'], $url);
			if($array['continue_flag'] == 1)
			{
				//error_log("Ceasing processing because flag = 1 (stop on match)");
				break;
			}
		}
	}

	//error_log("Transformed URL is $url");
	return $url;
}

function cookie_regexes($cookie)
{
	// Cookies contain a path that needs translating
	if(preg_match('path=',$cookie))
	{
		error_log("Mangling cookie $cookie");
		$index=strpos('path=',$cookie)+5;
		$left=substr($cookie,0,$index);
		$url=substr($cookie,$index);
		$url=perform_regexes($url);
		error_log("Cookie transformed to $url");
		$cookie=$left . $url;
	}
	return $cookie;
}

// Get going...

// URLt is not implemented as a "Tiki feature" yet. When it is, reinstate this code.
//if ($feature_urlt != 'y') {
//      $smarty->assign('msg', tra("This feature is disabled").": feature_urlt");
                                                                                                                             
//      $smarty->display("error.tpl");
//      die;
//}

// First, get the URL that we're translating. Normally this would be REQUEST_URI
// however, we allow an override for development/debugging/practice purposes
// The user can call tiki-urlt.php?url=... as well.
$url="";

if(isset($_REQUEST['url']))
{
	$url=$_REQUEST['url'];
}
else
{
	$url=$_SERVER['REQUEST_URI'];
}

# We have the URL to translate, so translate it...
$newurl=perform_regexes($url);

// We have to be careful to avoid request loops. That is, if we make a
// request to ourselves, it'll go around and around for ever. So, check to
// make sure we're not the object of their affections ;-)
if($newurl == $url)
{
	// Request loop!
	$smarty->assign('msg', tra("Warning: Request loop - $url translated to $newurl"));
	$smarty->display("error.tpl");
	die;
}

// Okay, we have a good looking URL - go fetch it...
// This requires that we check this URL for an actual file. We then figure out
// the MIME type of that file (unless it's a PHP) and then return it. If it's
// a PHP, then we simply include it, causing it to execute. We have to frig
// a few environmentals first though, so the PHP thinks everything is as it
// would normally be.
// Beware! URLs don't (necessarily) end in the file extension, e.g.:
// /tiki/tiki-index.php?page=banana

# Get the URI (ie the bit before the ? or #)
$uri=preg_replace('/(\?|#).*$/','',$url);

# Now see if the URI ends in a / - if so, then we artificially add
# index.php onto the end - this should probably be configurable!
if(preg_match('/\/$/',$uri))
{
	# This should be configurable!
	$uri=$uri . "index.php";
}

# Also do the same with the new url. When including the file, we don't
# want the query string (etc) on the URL
$newuri=preg_replace('/(\?|#).*$/','',$newurl);

//error_log("URLt: $url translated to $newurl");

# Construct the filename
# We should check to make sure there's no funny stuff going on here
# (ie. reading things outside the document root). NOT IMPLEMENTED!
$file=$_SERVER['DOCUMENT_ROOT'] . $newuri;

# Now check the file exists. If not, we have to return a 404 error.
if(!file_exists($file))
{
	# File doesn't exist. Return a 404 error (and let the web server
	# deal with that) NOT IMPLEMENTED
	$smarty->assign('msg', tra("Error 404: $file (translated from $url) does not exist"));
	$smarty->display("error.tpl");
	die;
}

if(preg_match('/\.php$/',$newuri))
{
	# Is a PHP file... Include it...
	$file=$_SERVER['DOCUMENT_ROOT'] . $newuri;
	# Now frig the environment ready for the PHP...
	# Setup everything so the included PHP doesn't know it's not been called directly
	$_SERVER['DOCUMENT_URI']=$newurl;
	if(strlen($newurl) > strlen($newuri))
	{
		# There is some kind of query string...
		# Snip it off, discard the ?
		$qs=substr($newurl,strlen($newuri)+1);
		# Since regexes arn't great at checking to see if we've got
		# two ? in the query string, we "fix" this here, by replacing
		# any ? with &. That makes any appended query strings work
		# nicely together.
		$qs=str_replace('?','&',$qs);
		# Now parse the QS and set $_REQUEST and $_GET
		$parts=array();
		$parts=explode('&',$qs);
		foreach ($parts as $part)
		{
			if($part == "")
			{
				continue;
			}
			$keyvalue=array();
			$keyvalue=explode('=',$part,2);
			//error_log("QS: For part $part Got key=$keyvalue[0] value=$keyvalue[1]");
			$_GET[$keyvalue[0]]=$keyvalue[1];
			$_REQUEST[$keyvalue[0]]=$keyvalue[1];
		}
	}
	else
	{
		$qs="";
	}
	$_SERVER['QUERY_STRING']=$qs;
	$_SERVER['DOCUMENT_NAME']=preg_replace('/\/([^\/*])$/','$1',$newuri);
	//error_log("Including $file: qs=" . $_SERVER['QUERY_STRING'] );
	#$smarty->assign('msg', tra("Information: PHP $url translated to $newurl (uri is $uri)"));
	include($file);
}
else
{
	# Non PHP - look up the MIME type and return the file
	$smarty->assign('msg', tra("Information: Non-PHP $url translated to $newurl (uri is $uri)"));
	$smarty->display("error.tpl");
	die;
}

?>
